From 0c54015a2264830ceb34f9458a4ebfbef92d7b70 Mon Sep 17 00:00:00 2001
From: Yuan Fu <casouri@gmail.com>
Date: Sat, 25 Jan 2020 17:27:32 -0500
Subject: [PATCH] Simplify update 1

---
 lisp/progmodes/gdb-mi.el | 19 ++++++++++++++++---
 lisp/progmodes/gud.el    |  6 +++++-
 2 files changed, 21 insertions(+), 4 deletions(-)

diff --git a/lisp/progmodes/gdb-mi.el b/lisp/progmodes/gdb-mi.el
index 27031e24f7..db1f544077 100644
--- a/lisp/progmodes/gdb-mi.el
+++ b/lisp/progmodes/gdb-mi.el
@@ -214,8 +214,6 @@ gdb-first-done-or-error
 (defvar gdb-source-window-list nil
   "List of windows used for displaying source files.
 Sorted in most recently visited first order.")
-(defvar gdb-max-source-window-count 2
-  "Maximum number of source windows to use.")
 (defvar gdb-inferior-status nil)
 (defvar gdb-continuation nil)
 (defvar gdb-supports-non-stop nil)
@@ -593,6 +591,21 @@ gdb-show-main
   :group 'gdb
   :version "22.1")
 
+(defcustom gdb-display-source-buffer-action nil
+  "`display-buffer' action used when GDB displaying a source buffer."
+  :type 'list
+  :group 'gdb
+  :version "28.1")
+
+(defcustom gdb-max-source-window-count 1
+  "Maximum number of source windows to use.
+Until there are such number of source windows on screen, GDB
+tries to open a new window when visiting a new source file; if
+there are, GDB will start to reuse existing source windows."
+  :type 'number
+  :grou 'gdb
+  :version "28.1")
+
 (defvar gdbmi-debug-mode nil
   "When non-nil, print the messages sent/received from GDB/MI in *Messages*.")
 
@@ -2020,7 +2033,7 @@ gdb-display-source-buffer
         (if (> gdb-max-source-window-count
                (length gdb-source-window-list))
             ;; create new window, push to list, return it
-            (car (push (display-buffer-pop-up-window buffer nil)
+            (car (push (display-buffer buffer gdb-display-source-buffer-action)
                        gdb-source-window-list))
           ;; reuse, we use the oldest window and put that to front
           (let ((last-win (car (last gdb-source-window-list)))
diff --git a/lisp/progmodes/gud.el b/lisp/progmodes/gud.el
index ccf9026417..8189394c60 100644
--- a/lisp/progmodes/gud.el
+++ b/lisp/progmodes/gud.el
@@ -2826,7 +2826,11 @@ gud-display-line
 	 (buffer
 	  (with-current-buffer gud-comint-buffer
 	    (gud-find-file true-file)))
-	 (window (when buffer (gdb-display-source-buffer buffer)))
+	 (window (when buffer (if (eq gud-minor-mode 'gdb-mi)
+                                  (gdb-display-source-buffer buffer)
+                                ;; we don't change the old behavior for gud
+                                (or (get-buffer-window buffer)
+                                    (display-buffer buffer '(nil (inhibit-same-window . t)))))))
 	 (pos))
     (when buffer
       (with-current-buffer buffer
-- 
2.25.0

